const db = require('_helpers/db');
const Address = db.useraddress;

module.exports = {
    getAll,
    getById,
    create,
    update,
    delete: _delete,
    getAllAddressesByUserId,
    setDefaultAddress,
    getDefaultAddress
};


async function getAll() {
    return await Address.find();
}

async function getById(id) {
    return await Address.findById(id);
}

async function getAllAddressesByUserId(id){
    return await Address.find({"userid" : id});
}

async function create(addressParam) {
    // validate
    let address = await Address.findOne({$and:[
        { userid : addressParam.userid },
        { tag : addressParam.tag }
    ]})

    if (address){
        addressParam.isdefault = address.isdefault; 
        update(address._id,addressParam);
    }
    else{
        addressParam.isdefault = false;
        address = new Address(addressParam);
    }
    
    // save address
    await address.save();

    return true;
}


async function setDefaultAddress(id, addressId) {
    const addresses = await Address.find({ userid :id });

    // validate
    if (!addresses) throw 'Address not found';

    addresses.forEach(function(address) {
        if(address.isdefault == true){
            address.isdefault = false;
            address.save();
        }   
      });

    await Address.findOneAndUpdate({_id:addressId},{isdefault:true});
    
    return true;
}

async function getDefaultAddress(userid){
    let address = await Address.findOne({$and:[
        { userid : userid },
        { isdefault : true }
    ]})

    if(!address) return false;
    return address;
}

async function update(id, addressParam) {
    const address = await Address.findById(id);

    // validate
    if (!address) throw 'Address not found';

    // copy addressParam properties to address	
    Object.assign(address, addressParam);
	
    await address.save();
}

async function _delete(id) {
    await Address.findByIdAndRemove(id);
}