﻿const express = require('express');
const router = express.Router();
const userService = require('./users.service');
const response = require("../../_helpers/response");


// routes
router.get('/', getAll);
router.post('/register', register);
router.post('/login', login);
router.post('/logout', logout);
router.post('/resetpassword',resetPassword);
router.get('/profile/:id', getUserProfile);
router.put('/profile/:id', updateUserProfile);
router.delete('/:id', deleteUser);
router.post('/email/:id', sendEmail);

module.exports = router;

function getUserProfile(req, res, next){
    userService.getById(req.params.id)
    .then(profile => profile ? res.json(response(true,"Profile",profile)) : res.status(401).json(response(false,"Cannot view Profile",[])))
    .catch(err => next(err)); 
}

function updateUserProfile(req, res, next){
    userService.update(req.params.id, req.body)
        .then(() => res.json(response(true,"Profile updated successfully",[])))
        .catch(err => next(err));
}

function resetPassword(req, res, next){
    userService.update(req.body.id, req.body)
        .then(() => res.json(response(true,"Password has been reset",[])))
        .catch(err => next(err));
}

function getAll(req, res, next) {
    userService.getAll()
        .then(users => res.json(
            response(true,"all users",users))
        )
        .catch(err => next(err));
}

function register(req, res, next) {
    userService.create(req.body)
        .then(() => res.json(response(true,"Successfully Registered",[])))
        .catch(err => next(res.status(401).json(response(false,err,[]))));
}

function login(req, res, next) {
    userService.authenticate(req.body)
        .then(user => user ? res.json(response(true,"Successfully logged in",user)) : res.status(401).json(response(false,"Username or Password is incorrect",[])))
        .catch(err => next(err));
}

function logout(req, res, next) {
    userService.update(req.body.id,{"isloggedin" : false})
        .then(() => res.json(response(true,"Successfully Logged Out",[])))
        .catch(err => next(err));
}

function deleteUser(req, res, next) {
    userService.delete(req.params.id)
        .then(() => res.json(response(true,"Successfully deleted",[])))
        .catch(err => next(err));
}

function sendEmail(req, res, next){
    userService.email(req.params.id, {})
        .then(() => res.json(response(true,"Email send successfully",[])))
        .catch(err => next(err));
}
